require 'spec_helper'

describe Saucy::Kiss::Observer do
  subject do
    Saucy::Kiss::Observer.new('abc123')
  end

  let(:session) { stub('session') }
  let(:request) { stub('request', :session => session) }
  let(:javascript_client) { stub('javascript client', :record => true) }
  let(:http_client) { stub('http client', :record => true) }
  let(:small_plan) { stub('plan', :name => 'Small', :price => 10, :trial => false) }
  let(:large_plan) { stub('plan', :name => 'Large', :price => 20, :trial => false) }
  let(:account) { stub('account', :plan => small_plan, :id => 12345) }

  before(:each) do 
    Snogmetrics::KissmetricsApi.stub(:new).and_return(javascript_client)
    Kissmetrics::HttpClient.stub(:new).and_return(http_client)
  end

  context "setting the Snogmetrics output strategy" do
    it "tells Snogmetrics to use console.log in development" do
      with_rails_env(:development) do
        Snogmetrics::KissmetricsApi.should_receive(:new).
          with(anything(), anything(), :console_log)

        subject.plan_viewed(:request => request, :plan => small_plan)
      end
    end

    it "tells Snogmetrics to append to _kmq in test" do
      with_rails_env(:test) do
        Snogmetrics::KissmetricsApi.should_receive(:new).
          with(anything(), anything(), :array)

        subject.plan_viewed(:request => request, :plan => small_plan)
      end
    end

    it "tells Snogmetrics to send to KISSmetrics in production" do
      with_rails_env(:production) do
        Snogmetrics::KissmetricsApi.should_receive(:new).
          with(anything(), anything(), :live)

        subject.plan_viewed(:request => request, :plan => small_plan)
      end
    end
  end

  context "Snogmetrics configuration" do
    it "tells Snogmetrics to use the correct API key" do
      Snogmetrics::KissmetricsApi.should_receive(:new).
        with('abc123', anything(), anything())

      subject.plan_viewed(:request => request, :plan => small_plan)
    end

    it "tells Snogmetrics to use the session" do
      Snogmetrics::KissmetricsApi.should_receive(:new).
        with(anything(), session, anything())

      subject.plan_viewed(:request => request, :plan => small_plan)
    end
  end

  it "records a Viewed Plan event through the KISSmetrics JS API for #plan_viewed" do
    javascript_client.should_receive(:record).
      with('Viewed Plan', { 'Plan Name' => 'Small', 'Plan Price' => 10, 'Is Trial Plan?' => false })

    subject.plan_viewed(:request => request, :plan => small_plan)
  end

  it "records a Viewed Plan List event through the KISSmetrics JS API for #plan_list_viewed" do
    javascript_client.should_receive(:record).
      with('Viewed Plan List')

    subject.plan_list_viewed(:request => request, :account => account)
  end

  it "records a Signed Up event through the KISSmetrics JS API for #account_created" do 
    javascript_client.should_receive(:record).
      with('Signed Up', { 'Plan Name' => 'Small', 'Plan Price' => 10, 'Is Trial Plan?' => false })

    subject.account_created(:request => request, :account => account)
  end

  it "records an Upgraded event through the KISSmetrics JS API for #plan_upgraded" do
    javascript_client.should_receive(:record).
      with('Upgraded', { 'Previous Plan Name' => 'Small', 'Plan Name' => 'Large', 'Plan Price' => 20, 'Is Trial Plan?' => false })

    subject.plan_upgraded(:request => request, :account => account, :from_plan => small_plan, :to_plan => large_plan)
  end

  it "records a Downgraded event through the KISSmetrics JS API for #plan_downgraded" do
    javascript_client.should_receive(:record).
      with('Downgraded', { 'Previous Plan Name' => 'Large', 'Plan Name' => 'Small', 'Plan Price' => 10, 'Is Trial Plan?' => false })

    subject.plan_downgraded(:request => request, :account => account, :from_plan => large_plan, :to_plan => small_plan)
  end

  it "records a Billed event through the KISSmetrics HTTP API for #billed" do
    Kissmetrics::HttpClient.should_receive(:new).
      with('abc123')

    account_identity = "account-#{account.id}"

    http_client.should_receive(:record).
      with(account_identity, 'Billed', { 'Billing Description' => 'Monthly billing for Small plan', 'Billing Amount' => 10 })

    subject.billed(:request => request, :account => account)
  end

  it "records a Canceled event through the KISSmetrics JS API for #canceled" do
    javascript_client.should_receive(:record).
      with('Canceled', { 'Plan Name' => 'Small', 'Plan Price' => 10, 'Is Trial Plan?' => false })

    subject.canceled(:request => request, :account => account)
  end

  it "records an Activated event through the KISSmetrics HTTP API for #activated" do
    Kissmetrics::HttpClient.should_receive(:new).
      with('abc123')

    account_identity = "account-#{account.id}"
    http_client.should_receive(:record).
      with(account_identity, 'Activated', { 'Plan Name' => 'Small', 'Plan Price' => 10, 'Is Trial Plan?' => false })

    subject.activated(:request => request, :account => account)
  end
end
